#
# Dockable Toolbar
# ----------------------------------------------------------------------
# This widget creates a toolbar frame which supports a collection of 
# configurable toolbars that can be docked, moved, etc.
# ----------------------------------------------------------------------
#  AUTHOR: Brian Griffin                 EMAIL: bgriffin@model.com
#
# ----------------------------------------------------------------------
#            Copyright © 2003 Model Technology Inc.
#                             a Mentor Grapics Company
# ======================================================================
# See the file "license.terms" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
#  Version: 0.2
#
# ======================================================================
# This widget has been rewritten in [snit], since there seems to be some 
# unfounded prejudice against incr, to include in the Tk Revitalization 
# effort.  --Brian

package require snit
package require Toolbar
package provide Dockbar 0.2

# ------------------------------------------------------------------
#                            DOCKBAR
# ------------------------------------------------------------------

option add *Dockbar.relief sunken widgetDefault
option add *Dockbar.borderWidth 1 widgetDefault

::snit::widget ::dockbar {

    widgetclass Dockbar

    #option {-height height Height} 0
    #option {-borderwidth borderWidth BorderWidth} 1
    #option {-relief relief Relief} sunken
    option {-propertycommand propertyCommand PropertyCommand} {} 
    option {-propertyprefix propertyPrefix PropertyPrefix} {} 

    delegate option * to hull

    # public method add {name args} 
    # public method hide {index}
    # public method index {index} 
    # public method itemcget {index args} 
    # public method itemconfigure {index args} 
    # public method itemname {index}
    # public method move {index x y}
    # public method redoPack {args}
    # public method remove {index} 
    # public method show {index}

    # protected method _geom_dragframe {X Y}
    # protected method _getMaxH {}
    # protected method _grab_b1 {w X Y}
    # protected method _grab_motion {w X Y}
    # protected method _grab_release {w X Y}
    # protected method _join {tb}
    # protected method _move {tbf x y}
    # protected method _post_configure_menu {}
    # protected method _rip {tb X Y}
    # protected method _tbConfigChange {tb}
    # protected method _toggleHide {tb}
    # protected method _inside {X Y w}
    # protected method setProperty {prop var}
    # protected method getProperty {prop var def}

    # private method _getAttachedOption {widget option retValueVar} 
    # private method _getShowVarValue {tb}
    # private method _setAttachedOption {widget option args} 
    # private method _setShowVariable {newvar oldvar}
    # private method _delayedRedoPack {args}
    # private method _delayedEval {args}

    # Components
    variable popup 
    variable components    ;# Components array

    variable _opts               ;# New options for child widgets
    variable _tbCnt 0
    variable _moveX 0
    variable _moveY 0
    variable _startX
    variable _startY
    variable _startH
    variable _startW
    variable _startdx
    variable _startdy
    variable _ordery
    variable hidden
    variable ripped
    variable names
    variable tbList [list]
    variable tbX
    variable tbY
    variable maxh 1
    variable _inRedoPack 0
    variable _delayedIds [list]

    #
    # Set _enableDragout if the patch for TIP 125 is present
    # This will enable the dragging out of a toolbar from the 
    # dockbar frame and allow it to "stand-alone".
    #
    typevariable _enableDragout
    typevariable _itemConfigs [list -showvariable]
    typevariable _itemConfigsOps [list \
		[list -showvariable showVariable ShowVariable] \
		]
    typevariable _showVars			;# Array of show variables and objects


    # ------------------------------------------------------------------
    #                            CONSTRUCTORs
    # ------------------------------------------------------------------
    typeconstructor {
	set _enableDragout [expr {[catch {wm toplevel .} ] == 0}]
    }

    constructor {args} {

	#
	# Popup Configure Menu
	#
	install popup using menu $self.popup -tearoff 0 

	$popup configure -postcommand [list $self _post_configure_menu]
	bind $self <ButtonPress-3> "tk_popup $popup %X %Y"

	#
	# Handle configs
	#
	$self configurelist $args

	bind $self <Visibility> [list $self _delayedRedoPack redoPack {Visibility Event}]
    }

    destructor {
	# Save toolbar layout
	foreach tb $tbList {
	    set name $names($tb)
	    $self setProperty "${name}_expelled" ripped($tb)
	    $self setProperty "${name}_hidden" hidden($tb)
	    $self setProperty "${name}_y" tbY($tb)
	    $self setProperty "${name}_x" tbX($tb)
	}
	# Cancel idle activity
	foreach aid $_delayedIds {
	    after cancel $aid
	}
	# Remove variable triggers
	foreach v [array names _showVars] {
	    while {[set n [lsearch -exact $_showVars($v) $self]] >= 0} {
		set _showVars($v) [lreplace $_showVars($v) $n $n]
	    }
	    if {[llength $_showVars($v)] == 0} {
		## Remove trace
		upvar #0 $v svar
		trace vdelete svar w [list $self redoPack]
		unset _showVars($v)
	    }
	}
    }

    # ------------------------------------------------------------------
    # PRIVATE METHOD: _setShowVariable
    #
    # Creates traces for the show variables--only one trace per variable.
    # The trace triggers a repacking of the toolbar which only need to
    # do once per variable change regardless of the number of times the
    # variable has been used.
    #
    # variable if 1 (TRUE) will be packed into toolbar.
    # If 0 (FALSE) will be unpacked from toolbar.
    # ------------------------------------------------------------------
    method _setShowVariable {newvar oldvar} {
	if {![info exists _showVars($newvar)]} {
	    set _showVars($newvar) [list]
	}
	lappend _showVars($newvar) $self
	if {[llength $_showVars($newvar)] == 1} {
	    ## Create trace on new variable
	    upvar #0 $newvar svar
	    trace variable svar w [list $self redoPack]
	}

	if {[info exists _showVars($oldvar)] && 
	    ([llength $_showVars($oldvar)] > 0) } {
	    set n [lsearch -exact $_showVars($oldvar) $self]
	    if {$n >= 0} {
		set _showVars($oldvar) [lreplace $_showVars($oldvar) $n $n]
	    }
	    if {[llength $_showVars($oldvar)] == 0} {
		## Remove trace on old variable
		trace vdelete svar w [list $self redoPack]
		unset _showVars($oldvar)
	    }
	}
    }

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # OPTION MANAGEMENT for -showvar
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # -------------------------------------------------------------
    # PRIVATE METHOD: _getAttachedOption
    #
    # widget     : the widget that the option is associated with
    # option     : the option whose value we are looking for on 
    #              this widget.
    #
    # expects to be called only if the $option is length 1
    # -------------------------------------------------------------
    method _getAttachedOption {widget option retValueVar} {
	upvar $retValueVar retValueRef
	set success false
	if { [info exists _opts($widget,$option)] } {
	    set retValueRef $_opts($widget,$option)
	    set success true
	}
	return $success
    }

    # -------------------------------------------------------------
    # PRIVATE METHOD: _setAttachedOption
    #
    # This method allows us to attach new options to a widget. It
    # catches the 'option' to be attached, strips it out of 'args'
    # attaches it to the 'widget' by stuffing the value into
    # 'optionList(widget,option)'
    #
    # widget: is the widget we want to associate the attached option
    # option: is the attached option (unknown to this widget)
    # args:   the arg list to search and remove the option from (if found)
    #
    # Modifies the args parameter.
    # Returns boolean indicating the success of the method
    #
    # -------------------------------------------------------------
    method _setAttachedOption {widget option args} {
	upvar args argsRef
    
	set success false
    
	# check for 'option' in the 'args' list for the 'widget'
	set optPos [eval lsearch -glob $args $option]
    
	# ... found it
	if { $optPos != -1 } {
	    # grab a copy of the option from arg list
	    set newOptValue [eval lindex $args [expr {$optPos + 1}]]
	    if {[info exists _opts($widget,$option)]} {
		set oldOptValue $_opts($widget,$option)
	    } else {
		set oldOptValue {}
	    }

	    # Special handling for showvariables. 
	    if {$option=="-showvariable" && ($newOptValue != $oldOptValue)} {
		$self _setShowVariable $newOptValue $oldOptValue
	    }

	    set _opts($widget,$option) $newOptValue

	    # remove the option argument and value from the arg list
	    set argsRef [eval lreplace $args $optPos [expr {$optPos + 1}]]
	    set success true
	}
	# ... if not found, will leave args alone
    
	return $success
    }

    #
    # Property access commands, use -propertycommand to
    # save and restore property values
    #
    method setProperty {prop var} {
	if {[$self cget -propertycommand] != ""} {
	    upvar $var data
	    if {[$self cget -propertyprefix] != ""} {
		set prop [format "%s%s" [$self cget -propertyprefix] $prop]
	    }
	    return [eval [$self cget -propertycommand] set $prop data]
	}
	return ""
    }

    method getProperty {prop var def} {
	upvar $var data
	if {[$self cget -propertycommand] != ""} {
	    if {[$self cget -propertyprefix] != ""} {
		set prop [format "%s%s" [$self cget -propertyprefix] $prop]
	    }
	    return [eval [$self cget -propertycommand] get $prop data $def]
	}
	set data $def
    }


    method add {name args} {
	#
	# Add a toolbar to the dockbar
	#

	set tbf [format "tbf%d" $_tbCnt]
	incr _tbCnt

	set components($tbf) ${self}.${tbf}

	install $tbf using frame $components($tbf) -relief raised -bd 1

	# Strip out and save the -showvariable options
	foreach c $_itemConfigs {
	    $self _setAttachedOption $components($tbf).$name $c $args
	}

	set maxh [$self _getMaxH]
	set maxy [lindex [lsort -integer -decreasing [array names _ordery]] 0]
	$self getProperty "${name}_y" newy 0

	$self getProperty "${name}_x" newx [llength $tbList]
	lappend tbList $tbf
	$self getProperty "${name}_hidden" hidden($tbf) 0
	set names($tbf) $name
	$self getProperty "${name}_expelled" ripped($tbf) 0

	#
	# create grabber
	#
	set gf [frame $components($tbf).gf -bd 0 -relief flat -cursor fleur]
	set g1 [frame ${gf}.g1 -width 3 -bd 1 -relief raised -cursor fleur]
	pack $g1 -side left -expand 0 -fill y -padx 1 -pady 1 -anchor nw
	pack $gf -side left -expand 0 -fill y -anchor nw -padx 1 -pady 0

	set components($name) $components($tbf).$name

	eval install $name using ::toolbar $components($tbf).$name $args -borderwidth 1 -wrap false

	# Override hard-coded relief of interior
	$components($name).tb configure -relief flat

	pack $components($name) -side left -fill both -expand 1 -anchor nw

	if {$ripped($tbf)} {
	    after idle [list $self _rip $tbf $newx $newy]
	} else {
	    $self _move $tbf $newx $newy
	}

	bind $g1 <Button-1> [list $self _grab_b1 $tbf %X %Y]
	bind $g1 <B1-Motion> [list $self _grab_motion $tbf %X %Y]
	bind $g1 <ButtonRelease-1> [list $self _grab_release $tbf %X %Y]
	bind $gf <Button-1> [list $self _grab_b1 $tbf %X %Y]
	bind $gf <B1-Motion> [list $self _grab_motion $tbf %X %Y]
	bind $gf <ButtonRelease-1> [list $self _grab_release $tbf %X %Y]
	$components($name) bind <Button-1> [list $self _grab_b1 $tbf %X %Y]
	$components($name) bind <B1-Motion> [list $self _grab_motion $tbf %X %Y]
	$components($name) bind <ButtonRelease-1> [list $self _grab_release $tbf %X %Y]

	bind $components($tbf) <Visibility> [list $self _delayedRedoPack redoPack [list Visibility Event on $tbf]]
	bind $components($tbf) <Configure> [list $self _delayedRedoPack redoPack [list Configure Event on $tbf]]
	bind $gf  <ButtonPress-3> "tk_popup $popup %X %Y"
	bind $g1  <ButtonPress-3> "tk_popup $popup %X %Y"
	bind $components($tbf) <ButtonPress-3> "tk_popup $popup %X %Y"

	return $components($name)
    }

    method remove {index} {
	set idx [$self index $index]
	set tb [lindex $tbList $idx]
	set tbList [lreplace $tbList $idx $idx]
	if {[info exists tbX($tb)]} {
	    unset tbX($tb)
	}
	if {[info exists tbY($tb)]} {
	    unset tbY($tb)
	}
	set widget $components($tb)
	if {[winfo exists $widget]} {
	    destroy $widget
	}
	if {[info exists names($tb)]} {
	    unset components($names($tb))
	    unset names($tb)
	}
	unset components($tb)
	$self redoPack
    }

    method _grab_b1 {c X Y} {
	set wid $components($c)
	set _moveX [expr {$X - [winfo rootx $wid]}]
	set _moveY [expr {$Y - [winfo rooty $wid]}]
	set top [winfo toplevel $wid]
	set geom [winfo geom $wid]
	_parseGeometry $geom _startW _startH _startX _startY xdir ydir
	if {$xdir == "-"} {
	    set _startX [expr {[winfo screenwidth $top] - ($_startW + $_startX)}]
	}
	if {$ydir == "-"} {
	    set _startY [expr {[winfo screenwidth $top] - ($_startH + $_startY)}]
	}
	set topgeom [wm geom $top]
	_parseGeometry $topgeom tW tH tX tY txdir tydir
	if {$txdir == "-"} {
	    set tX [expr {[winfo screenwidth $top] - ($tW + $tX)}]
	}
	if {$tydir == "-"} {
	    set tY [expr {[winfo screenwidth $top] - ($tH + $tY)}]
	}
	if {$ripped($c)} {
	    set _startdx [expr {$X - $tX}]
	    set _startdy [expr {$Y - $tY}]
	} else {
	    set _startdx [expr {$X - $tX - $_startX}]
	    set _startdy [expr {$Y - $tY - $_startY}]		
	}
	set com_opts [list -background grey50 -borderwidth 0 -relief flat -highlightthickness 0]
	eval toplevel $self._l -width 1 -height $_startH $com_opts
	eval toplevel $self._r -width 1 -height $_startH $com_opts
	eval toplevel $self._t -width $_startW -height 1 $com_opts
	eval toplevel $self._b -width $_startW -height 1 $com_opts
	wm transient $self._l [winfo toplevel $self]
	wm overrideredirect $self._l 1
	wm transient $self._r [winfo toplevel $self]
	wm overrideredirect $self._r 1
	wm transient $self._t [winfo toplevel $self]
	wm overrideredirect $self._t 1
	wm transient $self._b [winfo toplevel $self]
	wm overrideredirect $self._b 1
	$self _geom_dragframe [expr {$X - $_moveX}] [expr {$Y - $_moveY}]
    }

    method _geom_dragframe {X Y} {
	wm geom $self._l +$X+$Y
	wm geom $self._t +$X+$Y
	wm geom $self._r +[expr {$X + $_startW}]+$Y
	wm geom $self._b +$X+[expr {$Y + $_startH}]
    }

    method _grab_motion {c X Y} {
	set wid $components($c)
	if {[$self _inside  $X $Y $self]} {
	    $self._l configure -width 1
	    $self._r configure -width 1
	    $self._t configure -height 1
	    $self._b configure -height 1 -width [expr {$_startW + 1}]
	} else {
	    $self._l configure -width 3
	    $self._r configure -width 3
	    $self._t configure -height 3
	    $self._b configure -height 3 -width [expr {$_startW + 3}]
	}		
	$self _geom_dragframe [expr {$X - $_moveX}] [expr {$Y - $_moveY}]
    }

    method _inside {X Y w} {
	if {$_enableDragout} {
	    set fudge 12
	    set TX [winfo rootx $w]
	    set TY [winfo rooty $w]
	    _parseGeometry [winfo geom $w] w h x y xdir ydir
	    set lx [expr {$TX - $fudge}]
	    set rx [expr {$TX + $w + $fudge}]
	    set ty [expr {$TY - $fudge}]
	    set by [expr {$TY + (2 * $h) + $fudge}]
	    if {$X > $lx && $X < $rx && $Y > $ty && $Y < $by} {
		return 1
	    }
	} else {
	    return 1
	}
	return 0
    }

    method _grab_release {c X Y} {
	destroy $self._l
	destroy $self._r
	destroy $self._t
	destroy $self._b

	set widget $components($c)

	if {![$self _inside $X $Y $self]} {
	    # Outside
	    if {$ripped($c)} {
		# Geom the tb
		wm geom $widget [format "+%d+%d" [expr {$X - $_startdx}] [expr {$Y - $_startdy}]]
		set tbY($c) [expr {$Y - $_startdy}]
		set tbX($c) [expr {$X - $_startdx}]
	    } else {
		$self _rip $c [expr {$X - $_startdx}] [expr {$Y - $_startdy}]
	    }
	    return
	} else {
	    # Inside
	    if {$ripped($c)} {
		$self _join $c
	    }
	}
	set dy [winfo height $widget]
	set y [expr {(($Y - $_moveY - [winfo rooty $self])/$dy) * $dy}]
	if {$y < 0} {
	    set y 0
	}
	set x [expr {$X - $_moveX - [winfo rootx $self] - 1}]
	if {$x < 0} {set dx 0}
	$self _move $c $x [expr {($maxh > 0) ? ($y / $maxh) : 0}]
	return
    }

    method _rip {tb X Y} {
	set wid $components($tb)
	set ripped($tb) 1
	$self redoPack
	wm toplevel $wid 1
	set idx [lsearch -exact $tbList $tb]
	wm protocol $wid WM_DELETE_WINDOW [list $self hide $idx]
	wm resizable $wid 0 0
	wm transient $wid [winfo toplevel $self]
	wm title $wid [string totitle $names($tb)]
	set geom [format "+%d+%d" $X $Y]
	wm geom $wid $geom
	set tbY($tb) $Y
	set tbX($tb) $X
    }

    method _join {tb} {
	set wid $components($tb)
	wm toplevel $wid 0
	set ripped($tb) 0
    }

    method move {index x y} {
	set idx [$self index $index]
	set name [lindex $tbList $idx]
	$self _move $name $x [expr {($maxh > 0) ? ($y / $maxh) : 0}]
    }

    method _move {tbf x y} {
	if {[winfo ismapped $self]} {
	    set max [expr {[winfo width $self] - 16}]
	    if {$x > $max} {
		set x $max
	    }
	}
	set tbX($tbf) $x
	set tbY($tbf) $y
	$self redoPack
    }

    method _getShowVarValue {tb} {
	set widget $components($names($tb))
	if {[$self _getAttachedOption $widget {-showvariable} showVar]} {
	    upvar #0 $showVar svar
	    if {[info exists svar] && !$svar} {
		return 0
	    }
	}
	return 1
    }

    method _getMaxH {} {
	set maxh 0
	if {[info exists _ordery]} {unset _ordery}
	foreach tb $tbList {
	    if {$hidden($tb) || ![$self _getShowVarValue $tb]} {
		place forget $components($tb)
		continue
	    } elseif {$ripped($tb)} {
		continue
	    }
	    set h [winfo height $components($tb)]
	    set gridy $tbY($tb) ;#[expr {($tbY($tb)/$h) * $h}]
	    lappend _ordery($gridy) $tb
	    if {$h > $maxh} {
		set maxh $h
	    }
	}
	return $maxh
    }

    method redoPack {args} {
	if {$_inRedoPack} {
	    incr _inRedoPack
	    return
	}
	if {![winfo ismapped $self]} {
	    return
	}
	set _inRedoPack 1
	set maxh [$self _getMaxH]
	set lasty 0
	set realy 0
	foreach y [lsort -integer [array names _ordery]] {
	    if {[info exists orderx]} {unset orderx}
	    foreach tb $_ordery($y) {
		lappend orderx($tbX($tb)) $tb
	    }
	    set lastx 0
	    foreach x [lsort -integer [array names orderx]] {
		foreach tb $orderx($x) {
		    if {$x < $lastx} {
			set x [expr {$lastx + 1}]
		    }
		    set widget $components($tb)
		    place $widget -x $x -y $realy
		    update idletasks
		    set tbY($tb) [expr {$realy / $maxh}]
		    set lastx [expr {$x + [winfo reqwidth $widget]}]
		    if {[winfo reqheight $components($tb)] < $maxh} {
			set nh [expr {$maxh - (2 * [$widget cget -borderwidth])}]
			$widget.$names($tb) configure -minheight [expr {$nh - (2 * [$widget.$names($tb) cget -borderwidth])}]
			$widget.$names($tb) _configurePack
		    }
		}
	    }
	    incr realy $maxh
	}
	set h [expr {$realy + ([$self cget -borderwidth]*2)}]
	$self configure -height $h 
	if {$_inRedoPack > 1} {
	    $self _delayedRedoPack redoPack again
	}
	set _inRedoPack 0
    }



    method index {index} {
	if {[string is integer $index] && [llength $tbList] > $index} {
	    return $index
	} elseif {[info exists $index]} {
	    set idx [lsearch -exact $tbList [winfo name [winfo parent $components($index)]]]
	    return $idx
	} elseif {$index == "end"} {
	    return [expr {[llength $tbList] - 1}]
	} else {
	    return -1
	}
    }
		
    method hide {index} {
	if {[set idx [$self index $index]] >= 0} {
	    set tb [lindex $tbList $idx]
	    if {$ripped($tb)} {
		$self _join $tb
	    }
	    set hidden($tb) 1
	}
	$self redoPack
    }

    method _toggleHide {tb} {
	# This is called from checkbutton code where the
	# state variable has already been changed.
	# Call the appropriate routine to do the work.
	# (this means this code my look backwards, but it's not)
	set idx [lsearch -exact $tbList $tb]
	if {$hidden($tb)} {
	    $self hide $idx
	} else {
	    $self show $idx
	}
    }

    method show {index} {
	if {[set idx [$self index $index]] >= 0} {
	    set tb [lindex $tbList $idx]
	    set hidden($tb) 0
	    $components($names($tb)) redoPack
	}
	$self redoPack
    }

    method itemname {index} {
	if {[set idx [$self index $index]] >= 0} {
	    return $names([lindex $tbList $idx])
	}
    }

    method itemcget {index args} {
	return [lindex [eval itemconfigure $index $args] 4]
    }

    method itemconfigure {index args} {
	if {[set name [$self itemname $index]] != ""} {
	    set len [llength $args]
	    set widget $components($name)
	    switch $len {
		0 {
		    # Show all options

		    # support display of -showvariable configs
		    set optList [$widget configure]

		    foreach c $_itemConfigs cstr $_itemConfigsOps {
			$self _getAttachedOption $widget $c value
			set olist $cstr
			lappend olist $value
			lappend optList $olist
		    }
		    return $optList
		}
		1 {
		    # Show only option specified

		    foreach c $_itemConfigs cstr $_itemConfigsOps {
			if {[regexp -- $c $args]} {
			    $self _getAttachedOption $widget $c value
			    set olist $cstr
			    lappend olist {} $value
			    return $olist
			}
		    }
		    return [eval $widget configure $args]
		}
		default {
		    # ... do a normal configure

		    # First screen for all our child options we are adding
		    foreach c $_itemConfigs {
			$self _setAttachedOption $widget $c $args
		    }

		    # with a clean args list do a configure

		    # if the stripping process brought us down to no options,
		    # then forget the configure of widget.
		    if {[llength $args] != 0} {
			return [uplevel $widget configure $args]
		    } else {
			return ""
		    }
		}
	    }
	}
    }

    method itemcget {index args} {
	if {[set name [$self itemname $index]] != ""} {
	    uplevel $components($name) cget $args
	}
    }

    method _post_configure_menu {} {
	set menu $popup
	$menu delete 0 end
	foreach tb $tbList {
	    if {[$self _getShowVarValue $tb]} {
		set state normal
	    } else {
		set state disabled
	    }
	    $menu add checkbutton -label [string totitle $names($tb)] \
		-variable [varname hidden($tb)] \
		-command [list $self _toggleHide $tb] \
		-onvalue 0 -offvalue 1 -state $state
	    
	}
	$menu add separator
	$menu add command -label "Customize..." -command "echo Not yet implemented"
	$menu add separator
	$menu add command -label "Reset" -command [list $self reset]
    }

    #
    # We must keep track of idle events that we've scheduled so they
    # can be deleted before they mature when this object is deleted.
    #
    method _delayedRedoPack {args} {
	set cmd [concat list $self _delayedEval $args]
	set cmd [eval $cmd]
	set id [eval after idle $cmd]
	lappend _delayedIds $id
    }

    method _delayedEval {args} {
	set _delayedIds [lrange $_delayedIds 1 end]
	eval $self $args
    }

    method reset {} {
	set x 0
	foreach tb $tbList {
	    set name $names($tb)
	    set ripped($tb) 0
	    set tbY($tb) 0
	    set tbX($tb) $x
	    $self setProperty "${name}_expelled" ripped($tb)
	    $self setProperty "${name}_hidden" hidden($tb)
	    $self setProperty "${name}_y" tbY($tb)
	    $self setProperty "${name}_x" tbX($tb)
	    incr x
	}

	$self _delayedRedoPack redoPack reset
    }

    #
    # Utility proc
    #
    proc _parseGeometry {geom vw vh vx vy vxdir vydir} {
	upvar 1 $vw w
	upvar 1 $vh h
	upvar 1 $vx x
	upvar 1 $vy y
	upvar 1 $vxdir xdir
	upvar 1 $vydir ydir

	# Part One:  split into size and location parts
	# WxH | +X+Y 
	regexp -- {([0-9]+x[0-9]+)?([\+\-]+[0-9]+[\+\-]+[0-9]+)?} $geom foo size loc

	# Part II: process the two parts individually
	if {$size != ""} {
	    regexp {([0-9]+)x([0-9]+)} $size foo w h
	}
	if {$loc != ""} {
	    regexp {([\+\-])(-?[0-9]+)([\+\-])(-?[0-9]+)} $loc foo xdir x ydir y
	}
    }

} ;# end of ::dockbar

