#
# Toolbar
# ----------------------------------------------------------------------
#
#=====================================================================
# This toolbar widget was originally written by Bill Scott in [incrTk] 
# as part of the Iwidgets collection.  It was converted from using
# a [text] widget to using [place] in a [frame] for better wrapping
# characteristics.  It was then rewritten in [snit], since there seems
# to be some unfounded prejudice against incr, to include in the Tk
# Revitalization effort.  --Brian
#=====================================================================
#
# The Toolbar command creates a new window (given by the pathName 
# argument) and makes it into a Tool Bar widget. Additional options, 
# described above may be specified on the command line or in the 
# option database to configure aspects of the Toolbar such as its 
# colors, font, and orientation. The Toolbar command returns its 
# pathName argument. At the time this command is invoked, there 
# must not exist a window named pathName, but pathName's parent 
# must exist.
# 
# A Toolbar is a widget that displays a collection of widgets arranged 
# either in a row or a column (depending on the value of the -orient 
# option). This collection of widgets is usually for user convenience 
# to give access to a set of commands or settings. Any widget may be 
# placed on a Toolbar. However, command or value-oriented widgets (such 
# as button, radiobutton, etc.) are usually the most useful kind of 
# widgets to appear on a Toolbar.
#
# WISH LIST: 
#   This section lists possible future enhancements.  
#
#	Toggle between text and image/bitmap so that the toolbar could
#     display either all text or all image/bitmaps.
#   Implementation of the -toolbarfile option that allows toolbar
#     add commands to be read in from a file.
# ----------------------------------------------------------------------
#  AUTHOR: Bill W. Scott                 EMAIL: bscott@spd.dsccc.com
#
#  @(#) $Id: mtitoolbar.itk,v 1.36 2003/06/10 22:02:50 bgriffin Exp $
# ----------------------------------------------------------------------
#            Copyright (c) 1995 DSC Technologies Corporation
# ======================================================================
# Permission to use, copy, modify, distribute and license this software 
# and its documentation for any purpose, and without fee or written 
# agreement with DSC, is hereby granted, provided that the above copyright 
# notice appears in all copies and that both the copyright notice and 
# warranty disclaimer below appear in supporting documentation, and that 
# the names of DSC Technologies Corporation or DSC Communications 
# Corporation not be used in advertising or publicity pertaining to the 
# software without specific, written prior permission.
# 
# DSC DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING 
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, AND NON-
# INFRINGEMENT. THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND THE
# AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, 
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. IN NO EVENT SHALL 
# DSC BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR 
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
# SOFTWARE.
# ======================================================================
#
# Enhancements:
#
# Toolbar wraps -- if width of toolbar cannot fit all widgets, the
#                  toolbar will wrap the widgets onto a subsequent
#                  line.  This behavior is similar to the way the
#                  menubar wraps.
#
# Two item configure options added to the Add method:
#
#   -justify left|right    -- controls placement of the widgets
#
#   -showvariable varname  -- the named variable is a boolean which
#                             controls the visibility of the widget
#                             In other words, the widget will appear
#                             or disappear based on the state of this
#                             variable.
#
# Author: b.s.griffin       e-mail: bgriffin@model.com
#
# ----------------------------------------------------------------------
#            Copyright  2003 Mentor Graphics Corporation
# ======================================================================
# See the file "license.terms" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.

#
# Default resources.
#
option add *Toolbar.padX 1 widgetDefault
option add *Toolbar.padY 0 widgetDefault
option add *Toolbar.orient horizontal widgetDefault
option add *Toolbar*highlightThickness 0 widgetDefault
option add *Toolbar*indicatorOn false widgetDefault
option add *Toolbar*selectColor [. cget -bg] widgetDefault

#
# Usual options.
#
#itk::usual Toolbar {
#    keep -activebackground -activeforeground -background -balloonbackground \
#	-balloondelay1 -balloondelay2 -balloonfont -balloonforeground \
#	-borderwidth -cursor -disabledforeground -font -foreground \
#	-highlightbackground -highlightcolor -highlightthickness \
#	-insertbackground -insertforeground -selectbackground \
#	-selectborderwidth -selectcolor -selectforeground -troughcolor \
#	-relief -barfont -takefocus
#}

package require snit
package provide Toolbar 0.2

# ------------------------------------------------------------------
#                            TOOLBAR
# ------------------------------------------------------------------
::snit::widgetadaptor ::toolbar {

    #widgetclass Toolbar
    
    constructor {args} {}
    destructor {}
    
    option {-balloondelay1 balloonDelay1 BalloonDelay1} 1000
    option {-balloondelay2 balloonDelay2 BalloonDelay2} 200
    option {-barfont barfont Barfont} {Helvetica 10}
    option {-helpvariable helpVariable HelpVariable} {} 
    option {-minheight minheight MinHeight} 0
    option {-orient orient Orient} "horizontal" 
    option {-wrap wrap Wrap} true

    delegate option {-borderwidth borderWidth BorderWidth} to hull

    #
    # The following options implement propogated configurations to
    # any widget that might be added to us. The problem is this is
    # not deterministic as someone might add a new kind of widget with
    # and option like -armbackground, so we would not be aware of
    # this kind of option. Anyway we support as many of the obvious
    # ones that we can. They can always configure them with itemconfigures.
    #
    if {0} {
	# for now, comment these out since SNIT will (should) delegate
	# the options
	option {-activebackground activeBackground Foreground} #c3c3c3
	option {-activeforeground activeForeground Background} Black
	option {-background background Background} #d9d9d9 
	option {-borderwidth borderWidth BorderWidth} 2 
	option {-cursor cursor Cursor} {}
	option {-disabledforeground disabledForeground DisabledForeground} #a3a3a3 
	option {-font font Font} {Helvetica 9}
	option {-foreground foreground Foreground} #000000000000 
	option {-highlightbackground highlightBackground HighlightBackground} #d9d9d9 
	option {-highlightcolor highlightColor HighlightColor} Black 
	option {-highlightthickness highlightThickness HighlightThickness} 0 
	option {-insertforeground insertForeground Background} #c3c3c3 
	option {-insertbackground insertBackground Foreground} Black 
	option {-selectbackground selectBackground Foreground} #c3c3c3 
	option {-selectborderwidth selectBorderWidth BorderWidth} {} 
	option {-selectcolor selectColor Background} #b03060 
	option {-selectforeground selectForeground Background} Black 
	option {-state state State} normal 
	option {-takefocus takeFocus TakeFocus} 0
	option {-troughcolor troughColor Background} #c3c3c3 
    }

    # public method add {widgetCommand name args} 
    # public method bind {args}
    # public method delete {args} 
    # public method index {index} 
    # public method insert {beforeIndex widgetCommand name args} 
    # public method itemcget {index args} 
    # public method itemconfigure {index args} 
    # public method itemclass {index}
    # public method itemname {index}
    # public method move {srcIndex destIndex}
    # public method redoPack {args}

    # public method _resetBalloonTimer {}
    # public method _startBalloonDelay {window}
    # public method _stopBalloonDelay {window balloonClick}

    # private method _deleteWidgets {index1 index2} 
    # private method _addWidget {widgetCommand name args}
    # private method _index {toolList index} 
    # private method _getAttachedOption {widget option retValueVar} 
    # private method _setAttachedOption {widget option args} 
    # private method _packToolbar {} 
    # public method _configurePack {}
    # private method _setShowVariable {newvar oldvar}
    # public method _toolEnter {tool toolFrame}
    # public method _toolLeave {tool toolFrame}

    # public method hideHelp {} 
    # public method showHelp {window} 
    # public method showBalloon {window} 
    # public method hideBalloon {} 
    
    variable _balloonTimer 0
    variable _balloonAfterID 0
    variable _balloonClick false
    
    variable _interior {}
    variable _initialMapping 1   ;# Is this the first mapping?
    variable _toolList {}        ;# List of all widgets on toolbar
    variable _opts               ;# New options for child widgets
    variable _currHelpWidget {}  ;# Widget currently displaying help for
    variable _hintWindow {}      ;# Balloon help bubble.
    variable _hintWindowLabel {} ;# Balloon help label.
    variable _width 0            ;# Last packed widget width
    variable _cpId ""            ;# Configure changes background packing

    typevariable  _itemConfigs [list -helpstr -balloonstr -showvariable -justify]
    typevariable _itemConfigsOps [list \
		[list -helpstr helpStr HelpStr] \
		[list -balloonstr balloonStr BalloonStr] \
		[list -showvariable showVariable ShowVariable] \
		[list -justify justify Justify] \
		]
    typevariable _showVars	 ;# Array of show variables and objects

    # list of options we want to propogate to widgets added to toolbar.
    typevariable _optionList {
	-activebackground \
	    -activeforeground \
	    -background \
	    -borderwidth \
	    -cursor \
	    -disabledforeground \
	    -font \
	    -foreground \
	    -highlightbackground \
	    -highlightcolor \
	    -highlightthickness \
	    -insertbackground \
	    -insertforeground \
	    -selectbackground \
	    -selectborderwidth \
	    -selectcolor \
	    -selectforeground \
	    -state \
	    -takefocus \
	    -troughcolor \
	}


    # ------------------------------------------------------------------
    #                            CONSTRUCTOR 
    # ------------------------------------------------------------------
    constructor {args} {

	installhull using frame -borderwidth 0 -class Toolbar

	install _interior using frame ${self}.tb -relief groove -bd 2 \
	    -highlightthickness 0 -cursor {} -width 0 -height 1 
	bindtags $_interior $_interior
	pack $_interior -side top -fill both -expand 1 -anchor w -padx 0 -pady 0

	::bind $_interior <Configure> [list $self _configurePack]

	set _hintWindow [toplevel ${self}.balloonHintWindow]
	wm withdraw $_hintWindow
	wm overrideredirect $_hintWindow 1
	set _hintWindowLabel [label $_hintWindow.label \
				  -relief ridge \
				  -borderwidth 2]
	pack $_hintWindow.label -side top -expand 1 -fill x -padx 0 -pady 0

	#
	# Handle configs
	#
	$self configurelist $args

	# ... Attach help handler to this widget
	::bind toolbar-help-${hull} \
	    <Enter> "+[list $self showHelp %W]"
	::bind toolbar-help-${hull} \
	    <Leave> "+[list $self hideHelp]"
    
	# ... Set up Microsoft style balloon help display.
	set _balloonTimer [$self cget -balloondelay1]
	::bind $_interior \
	    <Leave> "+[list $self _resetBalloonTimer]"
	::bind toolbar-balloon-${hull} \
	    <Enter> "+[list $self _startBalloonDelay %W]"
	::bind toolbar-balloon-${hull} \
	    <Leave> "+[list $self _stopBalloonDelay %W false]"
	::bind toolbar-balloon-${hull} \
	    <Button-1> "+[list $self _stopBalloonDelay %W true]"

    }

    # ------------------------------------------------------------------
    #                           DESTURCTOR
    # ------------------------------------------------------------------
    destructor {
	if {$_cpId != 0} {after cancel $_cpId}
	if {$_balloonAfterID != 0} {after cancel $_balloonAfterID}
	foreach v [array names _showVars] {
	    while {[set n [lsearch -exact $_showVars($v) $this]] >= 0} {
		set _showVars($v) [lreplace $_showVars($v) $n $n]
	    }
	    if {[llength $_showVars($v)] == 0} {
		## Remove trace
		upvar #0 $v svar
		trace vdelete svar w [code $this redoPack]
		unset _showVars($v)
	    }
	}
    }

    # ------------------------------------------------------------------
    #                            OPTIONS
    # ------------------------------------------------------------------

    # ------------------------------------------------------------------
    # OPTION -background
    # ------------------------------------------------------------------
    delegate option -background to _interior

    # ------------------------------------------------------------------
    # OPTION -balloonbackground
    # ------------------------------------------------------------------
    delegate option -balloonbackground to _hintWindowLabel as -background

    # ------------------------------------------------------------------
    # OPTION -balloonforeground
    # ------------------------------------------------------------------
    delegate option -ballonforeground to _hintWindowLabel as -foreground

    # ------------------------------------------------------------------
    # OPTION -balloonfont
    # ------------------------------------------------------------------
    delegate option -balloonfont to _hintWindowLabel as -font

    # ------------------------------------------------------------------
    # OPTION -barfont
    # ------------------------------------------------------------------

    # ------------------------------------------------------------------
    # OPTION: -orient
    #
    # Position buttons either horizontally or vertically.
    # ------------------------------------------------------------------
    onconfigure -orient {value} {
	switch $value {
	    "horizontal" - "vertical" {
		set options(-orient) $value
		$self _packToolbar
	    }
	    default {error "Invalid orientation. Must be either \
		horizontal or vertical"
	    }
	}
    }

    # ------------------------------------------------------------------
    #                            METHODS
    # ------------------------------------------------------------------
    
    # -------------------------------------------------------------
    # METHOD: add widgetCommand name ?option value?
    #
    # Adds a widget with the command widgetCommand whose name is 
    # name to the Toolbar.   If widgetCommand is radiobutton 
    # or checkbutton, its packing is slightly padded to match the 
    # geometry of button widgets.
    # -------------------------------------------------------------
    method add { widgetCommand name args } {

	set cname [eval "$self _addWidget $widgetCommand $name $args"]
    
	lappend _toolList $cname
    
	switch $widgetCommand {
	    "radiobutton" -
	    "checkbutton" {
		set iPad 1
	    }
	    default {
		set iPad 0
	    }
	}
    
	# repack the tool bar
	$self _packToolbar

	return $cname
    
    }

    # -------------------------------------------------------------
    #
    # METHOD: bind ?args?
    #
    # This command creates event bindings on the hull
    # args follows the same syntax as the bind command
    #
    # -------------------------------------------------------------
    method bind {args} {
	eval ::bind $_interior $args
    }

    # -------------------------------------------------------------
    #
    # METHOD: delete index ?index2?
    #
    # This command deletes all components between index and 
    # index2 inclusive. If index2 is omitted then it defaults 
    # to index. Returns an empty string
    #
    # -------------------------------------------------------------
    method delete { args } {
	# empty toolbar
	if { $_toolList == {} } {
	    error "can't delete widget, no widgets in the Toolbar \
		\"${hull}\""
	}
    
	set len [llength $args]
	switch -- $len {
	    1 {
		set fromWidget [_index $_toolList [lindex $args 0]]
	    
		if { $fromWidget < 0 || $fromWidget >= [llength $_toolList] } {
		    error "bad Toolbar widget index in delete method: \
			should be between 0 and [expr {[llength $_toolList] - 1}]"
		}
	    
		set toWidget $fromWidget
		$self _deleteWidgets $fromWidget $toWidget
	    }
	
	    2 {
		set fromWidget [_index $_toolList [lindex $args 0]]
	    
		if { $fromWidget < 0 || $fromWidget >= [llength $_toolList] } {
		    error "bad Toolbar widget index1 in delete method: \
			should be between 0 and [expr {[llength $_toolList] - 1}]"
		}
	    
		set toWidget [_index $_toolList [lindex $args 1]]
	    
		if { $toWidget < 0 || $toWidget >= [llength $_toolList] } {
		    error "bad Toolbar widget index2 in delete method: \
			should be between 0 and [expr {[llength $_toolList] - 1}]"
		}
		
		if { $fromWidget > $toWidget } {
		    error "bad Toolbar widget index1 in delete method: \
			index1 is greater than index2"
		}
	    
		$self _deleteWidgets $fromWidget $toWidget
	    }
	
	    default {
		# ... too few/many parameters passed
		error "wrong # args: should be \
		    \"${hull} delete index1 ?index2?\""
	    }
	}
    
	$this redoPack

	return {}
    }


    # -------------------------------------------------------------
    #
    # METHOD: index index 
    #
    # Returns the widget's numerical index for the entry corresponding 
    # to index. If index is not found, -1 is returned
    #
    # -------------------------------------------------------------
    method index { index } {
    
	return [$self _index $_toolList $index]
    
    }

    # -------------------------------------------------------------
    #
    # METHOD: insert beforeIndex widgetCommand name ?option value?
    #
    # Insert a new component named name with the command 
    # widgetCommand before the com ponent specified by beforeIndex. 
    # If widgetCommand is radiobutton or checkbutton, its packing 
    # is slightly padded to match the geometry of button widgets.
    #
    # -------------------------------------------------------------
    method insert { beforeIndex widgetCommand name args } {
    
	set beforeIndex [$self _index $_toolList $beforeIndex]
    
	if {$beforeIndex < 0 || $beforeIndex > [llength $_toolList] } {
	    error "bad toolbar entry index $beforeIndex"
	}
    
	set cname [eval "$self _addWidget $widgetCommand $name $args"]
    
	# linsert into list
	set _toolList [linsert $_toolList $beforeIndex $cname]
    
	# repack the tool bar
	$self _packToolbar
    
	return $cname
    
    }

    # ----------------------------------------------------------------------
    # METHOD: itemcget index ?option? 
    #
    # Returns the value for the option setting of the widget at index $index.
    # index can be numeric or widget name
    #
    # ----------------------------------------------------------------------
    method itemcget { index args} {
	
	return [lindex [eval $self itemconfigure $index $args] 4]
    }

    # -------------------------------------------------------------
    #
    # METHOD: itemconfigure index ?option? ?value? ?option value...?
    #
    # Query or modify the configuration options of the widget of 
    # the Toolbar specified by index. If no option is specified, 
    # returns a list describing all of the available options for 
    # index (see Tk_ConfigureInfo for information on the format 
    # of this list). If option is specified with no value, then 
    # the command returns a list describing the one named option 
    # (this list will be identical to the corresponding sublist 
    # of the value returned if no option is specified). If one 
    # or more option-value pairs are specified, then the command 
    # modifies the given widget option(s) to have the given 
    # value(s); in this case the command returns an empty string. 
    # The component type of index determines the valid available options.
    #
    # -------------------------------------------------------------
    method itemconfigure { index args } {
    
	# Get a numeric index.
	set index [$self _index $_toolList $index]

	# Get the tool path
	set toolPath [lindex $_toolList $index]
    
	set len [llength $args]
    
	switch $len {
	    0 {
		# show all options
		# ''''''''''''''''
		
		# support display of -helpstr and -balloonstr configs
		set optList [$toolPath configure]
		
		## @@@ might want to use _getAttachedOption instead...
		foreach c $_itemConfigs cstr $_itemConfigsOps {
		    if { [info exists _opts($toolPath,$c)] } {
			set value $_opts($toolPath,$c)
		    } else {
			set value {}
		    }
		    set olist $cstr
		    lappend olist {} $value
		    lappend optList $olist
		}
		return $optList
	    }
	    1 {
		# show only option specified
		# ''''''''''''''''''''''''''
		# did we satisfy the option get request?
		
		foreach c $_itemConfigs cstr $_itemConfigsOps {
		    if { [regexp -- $c $args] } {
			if { [info exists _opts($toolPath,$c)] } {
			    set value $_opts($toolPath,$c)
			} else {
			    set value {}
			}
			set olist $cstr
			lappend olist {} $value
			return $olist
		    }
		}
		return [eval $toolPath configure $args]
	    }
	    default {
		# ... do a normal configure
	    
		# first screen for all our child options we are adding
		foreach c $_itemConfigs {
		    $self _setAttachedOption $toolPath $c $args 
		}
		
		# with a clean args list do a configure
	    
		# if the stripping process brought us down to no options
		# to set, then forget the configure of widget.
		if { [llength $args] != 0 } {
		    return [eval $toolPath configure $args]
		} else {
		    return ""
		}
	    }
	}
	
    }

    # ----------------------------------------------------------------------
    # METHOD: itemclass index
    #
    # Returns the class for the item at index
    # index can be numeric or widget name
    #
    # ----------------------------------------------------------------------
    method itemclass { index} {
	set idx [_index $_toolList $index]
	set name [lindex $_toolList $idx]
	return [winfo class $name]
    }

    # ----------------------------------------------------------------------
    # METHOD: itemname index
    #
    # Returns the widget name for the item at index
    # index can be numeric or widget name
    #
    # ----------------------------------------------------------------------
    method itemname { index} {
	set idx [_index $_toolList $index]
	set name [lindex $_toolList $idx]
	return [winfo name $name]
    }

    # ----------------------------------------------------------------------
    # METHOD: itemname index
    #
    # Returns the widget name for the item at index
    # index can be numeric or widget name
    #
    # ----------------------------------------------------------------------
    method move {srcIndex destIndex} {
	set si [_index $_toolList $srcIndex]
	set di [_index $_toolList $destIndex]
	set sw [lindex $_toolList $si]

	set _toolList [lreplace $_toolList $si $si]
	set _toolList [linsert $_toolList $di $sw]

	$self _packToolbar
    }

    # -------------------------------------------------------------
    #
    # METHOD: _resetBalloonDelay1 
    #
    # Sets the delay that will occur before a balloon could be popped
    # up to balloonDelay1
    #
    # -------------------------------------------------------------
    method _resetBalloonTimer {} {
	set _balloonTimer [$self cget -balloondelay1]
    
	# reset the <1> longer delay
	set _balloonClick false
    }

    # -------------------------------------------------------------
    #
    # METHOD: _startBalloonDelay 
    #
    # Starts waiting to pop up a balloon id
    #
    # -------------------------------------------------------------
    method _startBalloonDelay {window} {
	set _balloonAfterID [after $_balloonTimer catch \{[list $self showBalloon $window]\}]
    }

    # -------------------------------------------------------------
    #
    # METHOD: _stopBalloonDelay  
    #
    # This method will stop the timer for a balloon popup if one is
    # in progress. If however there is already a balloon window up
    # it will hide the balloon window and set timing to delay 2 stage.
    #
    # -------------------------------------------------------------
    method _stopBalloonDelay { window balloonClick } {
    
	# If <1> then got a click cancel
	if { $balloonClick } {
	    set _balloonClick true
	}
	if { $_balloonAfterID != 0 } {
	    after cancel $_balloonAfterID
	    set _balloonAfterID 0
	} else {
	    $self hideBalloon
	
	    # If this was cancelled with a <1> use longer delay.
	    if { $_balloonClick } {
		set _balloonTimer [$self cget -balloondelay1]
	    } else {
		set _balloonTimer [$self cget -balloondelay2]
	    }
	}
    }

    # -------------------------------------------------------------
    # PRIVATE METHOD: _addWidget
    #
    # widgetCommand : command to invoke to create the added widget
    # name          : name of the new widget to add
    # args          : options for the widget create command
    #
    # Looks for -helpstr, -balloonstr and grabs them, strips from
    # args list. Then tries to add a component and keeps based
    # on known type. If it fails, it tries to clean up. Then it
    # binds handlers for helpstatus and balloon help.
    #
    # Returns the path of the widget added.
    #
    # -------------------------------------------------------------
    method _addWidget { widgetCommand name args } {
	# ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	# Add the widget to the tool bar
	# '''''''''''''''''''''''''''''''''''''''''''''''''''''

	# ... Strip out and save the -helpstr, -balloonstr options from args
	#     and save it in _opts

	foreach c $_itemConfigs {
	    $self _setAttachedOption $_interior.$name $c $args 
	}

	# ... Add the new widget as a component (catch an error if occurs)
	if {[catch {eval $widgetCommand $_interior.$name $args} errMsg]} {
	    # ... Clean up if the create failed, and exit.
	    #     The _opts list if it has -helpstr, -balloonstr just entered for
	    #     this, it must be cleaned up.
	    foreach c $_itemConfigs {
		if {[info exists _opts($_interior.$name,$c)]} {
		    set lastIndex [expr {[llength $_opts($_interior.$name,$c)]-1}]
		    lreplace $_opts($_interior.$name,$c) $lastIndex $lastIndex ""
		}
	    }
	    error $errMsg
	}

	variable $name
	set $name $errMsg
	set comp $errMsg
#TODO	delegate option -background to $comp
#TODO	delegate option -foreground to $comp

	# Now make the component
	# do we need to install the component here?

	# ... Add in dynamic options that apply from the _optionList
	foreach optionSet [$comp configure] {
		set option [lindex $optionSet 0]
		if { [lsearch $_optionList $option] != -1 } {
			#puts "do something with itk_option add $name.$option"
		}
	}

	bindtags $comp [linsert [bindtags $comp] end toolbar-help-$hull]
	bindtags $comp [linsert [bindtags $comp] end toolbar-balloon-$hull]
    
	return $comp
    }

    # -------------------------------------------------------------
    #
    # PRIVATE METHOD: _deleteWidgets
    #
    # deletes widget range by numerical index numbers.
    #
    # -------------------------------------------------------------
    method _deleteWidgets { index1 index2 } {
    
	for { set index $index1 } { $index <= $index2 } { incr index } {
	
	    # kill the widget
	    set component [lindex $_toolList $index]
	    destroy $component
	    if {[winfo exists $_interior.f_$index]} {
		destroy $_interior.f_$index
	    }
	}
	
	# physically remove the page
	set _toolList [lreplace $_toolList $index1 $index2]
	
    }

    # -------------------------------------------------------------
    # PRIVATE METHOD: _index
    #
    # toolList : list of widget names to search thru if index 
    #            is non-numeric
    # index    : either number, 'end', 'last', or pattern
    #
    # _index takes takes the value $index converts it to
    # a numeric identifier. If the value is not already
    # an integer it looks it up in the $toolList array.
    # If it fails it returns -1
    #
    # -------------------------------------------------------------
    method _index { toolList index } {
	switch -- $index {
	    end - last {
		set number [expr {[llength $toolList] -1}]
	    }
	    default {
		# is it a number already? Then just use the number
		if { [regexp {^[0-9]+$} $index] } {
		    set number $index
		    # check bounds
		    if { $number < 0 || $number >= [llength $toolList] } {
			set number -1
		    }
		    # otherwise it is a widget name
		} else {
		    set number [lsearch -glob $toolList "*${index}*"]
		}
	    }
	}
	return $number
    }
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # STATUS HELP for linking to helpVariable
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # -------------------------------------------------------------
    # 
    # PUBLIC METHOD: hideHelp
    #
    # Bound to the <Leave> event on a toolbar widget. This clears the
    # status widget help area and resets the help entry.
    #
    # -------------------------------------------------------------
    method hideHelp {} {
	if { [$self cget -helpvariable] != {} } {
	    upvar #0 [$self cget -helpvariable] helpvar
	    set helpvar {}
	}
	set _currHelpWidget {}
    }

    # -------------------------------------------------------------
    # 
    # PUBLIC METHOD: showHelp
    #
    # Bound to the <Motion> event on a tool bar widget. This puts the
    # help string associated with the tool bar widget into the 
    # status widget help area. If no help exists for the current
    # entry, the status widget is cleared.
    #
    # -------------------------------------------------------------
    method showHelp { window } {
	set widgetPath $window
	# already on this item?
	if { $window == $_currHelpWidget } {
	    return
	}
    
	set _currHelpWidget $window
    
	# Do we have a helpvariable set on the toolbar?
	if { [$self cget -helpvariable] != {} } {
	    upvar #0 [$self cget -helpvariable] helpvar
	    
	    # is the -helpstr set for this widget?
	    if {[$self _getAttachedOption $window {-helpstr} value]} {
		set helpvar $value.
	    } else {
		set helpvar {}
	    }
	}
    }

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # BALLOON HELP for show/hide of hint window
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # -------------------------------------------------------------
    # 
    # PUBLIC METHOD: showBalloon
    #
    # -------------------------------------------------------------
    method showBalloon {window} {
	set _balloonClick false
	set _balloonAfterID 0

	if {![winfo exists $window]} {
	    # This happens when application exits
	    return
	}

	# Are we still inside the window?
	set xp [winfo pointerx .]
	set yp [winfo pointery .]
	set mouseWindow [winfo containing $xp $yp]

	if { [string match $window* $mouseWindow] } {
	    if {$_hintWindow == {} || ![winfo exists $_hintWindow]} {
		# build balloon help window
		if {$_hintWindow != {}} {
		    puts "Where did $_hintWindow go!\n"
		}
	    }

	    # set up the balloonString
	    if {[$self _getAttachedOption $window {-balloonstr} hintStr]} {
		# configure the balloon help
		$_hintWindow.label configure -text $hintStr		

		set font [$_hintWindow.label cget -font]
		set bd   [$_hintWindow.label cget -borderwidth]
		set width [expr {[font measure $font $hintStr] + (2 * $bd) + 10}]
		set right [winfo screenwidth .]

		set height [expr {[font metrics $font -linespace] + (2 * $bd)}]

		incr xp 8 
		incr yp 16 
		if {[expr {$xp + $width}] > $right} {
		    set xp [expr {$right - $width}]
		}

		# put up balloon window
		wm withdraw $_hintWindow
		wm overrideredirect $_hintWindow 0
		wm geometry $_hintWindow "${width}x${height}+${xp}+${yp}"
		wm overrideredirect $_hintWindow 1
		wm deiconify $_hintWindow
		raise $_hintWindow
	    } else {
		#NO BALLOON HELP AVAILABLE
	    }
	} else {
	    #NOT IN BUTTON
	}
    }

    # -------------------------------------------------------------
    # 
    # PUBLIC METHOD: hideBalloon
    #
    # -------------------------------------------------------------
    method hideBalloon {} {
	if {[winfo exists $_hintWindow]} {
	    wm withdraw $_hintWindow
	}
    }

    # ------------------------------------------------------------------
    # PRIVATE METHOD: _setShowVariable
    #
    # Creates traces for the show variables--only one trace per variable.
    # The trace triggers a repacking of the toolbar which only need to
    # do once per variable change regardless of the number of times the
    # variable has been used.
    #
    # variable if 1 (TRUE) will be packed into toolbar.
    # If 0 (FALSE) will be unpacked from toolbar.
    # ------------------------------------------------------------------
    method _setShowVariable {newvar oldvar} {
	if {![info exists _showVars($newvar)]} {
	    set _showVars($newvar) [list]
	}
	lappend _showVars($newvar) $self
	if {[llength $_showVars($newvar)] == 1} {
	    ## Create trace on new variable
	    upvar #0 $newvar svar
	    trace variable svar w [list $self redoPack]
	}

	if {[info exists _showVars($oldvar)] && 
	    ([llength $_showVars($oldvar)] > 0) } {
	    set n [lsearch -exact $_showVars($oldvar) $this]
	    if {$n >= 0} {
		set _showVars($oldvar) [lreplace $_showVars($oldvar) $n $n]
	    }
	    if {[llength $_showVars($oldvar)] == 0} {
		## Remove trace on old variable
		trace vdelete svar w [list $self redoPack]
		unset _showVars($oldvar)
	    }
	}
    }


    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # OPTION MANAGEMENT for -helpstr, -balloonstr
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # -------------------------------------------------------------
    # PRIVATE METHOD: _getAttachedOption
    #
    # widget     : the widget that the option is associated with
    # option     : the option whose value we are looking for on 
    #              this widget.
    #
    # expects to be called only if the $option is length 1
    # -------------------------------------------------------------
    method _getAttachedOption {widget option retValueVar} {
	upvar $retValueVar retValueRef
	set success false
	if { [info exists _opts($widget,$option)] } {
	    set retValueRef $_opts($widget,$option)
	    set success true
	}
	return $success
    }

    # -------------------------------------------------------------
    # PRIVATE METHOD: _setAttachedOption
    #
    # This method allows us to attach new options to a widget. It
    # catches the 'option' to be attached, strips it out of 'args'
    # attaches it to the 'widget' by stuffing the value into
    # 'optionList(widget,option)'
    #
    # widget: is the widget we want to associate the attached option
    # option: is the attached option (unknown to this widget)
    # args:   the arg list to search and remove the option from (if found)
    #
    # Modifies the args parameter.
    # Returns boolean indicating the success of the method
    #
    # -------------------------------------------------------------
    method _setAttachedOption {widget option args} {
    
	upvar args argsRef
    
	set success false
    
	# check for 'option' in the 'args' list for the 'widget'
	set optPos [eval lsearch -glob $args $option]
    
	# ... found it
	if { $optPos != -1 } {
	    # grab a copy of the option from arg list
	    set newOptValue [eval lindex $args [expr {$optPos + 1}]]
	    if {[info exists _opts($widget,$option)]} {
		set oldOptValue $_opts($widget,$option)
	    } else {
		set oldOptValue {}
	    }

	    # Special handling for showvariables. 
	    if {$option=="-showvariable" && ($newOptValue != $oldOptValue)} {
		_setShowVariable $newOptValue $oldOptValue
	    }

	    set _opts($widget,$option) $newOptValue

	    # remove the option argument and value from the arg list
	    set argsRef [eval lreplace $args $optPos [expr {$optPos + 1}]]
	    set success true
	}
	# ... if not found, will leave args alone
    
	return $success
    }

    method _toolEnter {tool toolFrame} {
	if {[winfo exists $tool] && [winfo exists $toolFrame]} {
	    if {![catch {set state [$tool cget -state]}]} {
		if { $state != "disabled" } {
		    $toolFrame configure -relief raised
		}
	    }
	}
    }

    method _toolLeave {tool toolFrame} {
	if {[winfo exists $tool] && [winfo exists $toolFrame]} {
	    $toolFrame configure -relief flat 
	}
    }

    # -------------------------------------------------------------
    # 
    # PUBLIC METHOD: redoPack
    #
    #   redo the toolbar packing
    #
    # -------------------------------------------------------------
    method redoPack {args} {
	$self _packToolbar
    }

    # -------------------------------------------------------------
    # 
    # PUBLIC METHOD: _configurePack
    #
    #   redo the toolbar packing
    #
    # -------------------------------------------------------------
    method _configurePack {} {
	after cancel $_cpId
	set _cpId [after idle [list $self _packToolbar]]
	set _width [winfo width $_interior]
    }

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # GEOMETRY MANAGEMENT for tool widgets
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # -------------------------------------------------------------
    # 
    # PRIVATE METHOD: _packToolbar
    #
    #
    #
    # -------------------------------------------------------------
    method _packToolbar {} {

	if {![winfo ismapped $_interior]} {
	    return
	}
	if {[$self cget -orient] == "vertical"} {
	    set colInc 0
	    set rowInc 1
	} else {
	    set colInc 1
	    set rowInc 0
	}

	# pack in order of _toolList.
	set idx 0
	set row 0
	set col 0
	set width [expr {[winfo width $_interior] - (2 * [$_interior cget -bd])}]
	set leftExt 0
	set maxwidth 0
	set htotal 0
	set maxHt 0
	set repeat 0
	set rightExt $width
	#
	# Place all the tools in the toolbar
	foreach tool $_toolList {

	    # First determine if this tool should be packed at all.
	    if { [$self _getAttachedOption $tool {-showvariable} showVar] } {
		upvar #0 $showVar svar
		if {[info exists svar] && ($svar == 0)} {
		    continue
		} elseif {![winfo ismapped $tool]} {
		    # This tool is not mapped, so need
		    # to repeat packing since required size
		    # information will not be correct
		    set repeat 1
		}
	    }

	    # adjust for radios and checks to match buttons
	    if { [winfo class $tool] == "Radiobutton" || 
		 [winfo class $tool] == "Checkbutton" } {
		set iPad 1
	    } else {
		set iPad 0
	    }
		
	    # Determine tool justification
	    if { [$self _getAttachedOption $tool {-justify} justVar] } {
		if {$justVar == "right" || $justVar == "bottom"} {
		    #continue
		    set justifyRight 1
		}
	    } else {
		# default value
		set justifyRight 0
	    }

	    if {[winfo exists $_interior.f_$idx]} {
		set f $_interior.f_$idx
	    } else {
		set f [frame $_interior.f_$idx \
			   -borderwidth 1 \
			   -highlightthickness 0 \
			   -background [$self cget -background]]
	    }
	    # pack by horizontal or vertical orientation
	    pack $tool -in $f -side top -expand 0 -fill both -padx $iPad -pady $iPad
	    if {[$self cget -wrap] && $colInc && !$rowInc} {
		# Wrap if horizontal and extends beyond edge of widget
		if { $justifyRight } {
		    if {$leftExt > [expr {$rightExt - [winfo reqwidth $f]}]} {
			set col 0
			incr row
			set leftExt 0
			set rightExt $width
		    }
		} else {
		    if {[expr {[winfo reqwidth $f] + $leftExt}] > $width} {
			set col 0
			incr row
			set leftExt 0
			set rightExt $width
		    }
		}
	    }
	    if {[winfo reqheight $f] > $maxHt} {
		set maxHt [winfo reqheight $f]
	    }
	    if {[winfo reqwidth $f] > $maxwidth} {
		set maxwidth [winfo reqwidth $f]
	    }
	    if {$colInc && $justifyRight} {
		set x [expr {$rightExt - [winfo reqwidth $f] - 1}]
		set rightExt $x
	    } else {
		set x [expr {$leftExt + 2}]
	    }
	    set y [expr {($row * $maxHt) + (($maxHt - [winfo reqheight $f]) / 2) + 1}]
	    place $f -y $y -x $x

	    raise $tool $f
	    ::bind $tool <Enter> [list $self _toolEnter $tool $f]
	    ::bind $tool <Leave> [list $self _toolLeave $tool $f]
	    incr idx
	    incr row $rowInc
	    incr col $colInc
	    if {$colInc && !$justifyRight} {
		incr leftExt [winfo reqwidth $f]
	    }
	    if {$rowInc} {
		incr htotal [winfo reqheight $f]
	    }
	}
	if {[$self cget -wrap]} {
	    set hgt [expr {(($row + 1) * $maxHt) + 6}]
	    if {$hgt < [$self cget -minheight]} {
		set hgt [$self cget -minheight]
	    }
	    if {$colInc} {
		$_interior configure -height $hgt -width 0
	    } else {
		$_interior configure \
		    -height 0 \
		    -width [expr {$maxwidth + (2 * [$_interior cget -borderwidth]) + 6}]
	    }
	} else {
	    set hgt [expr {(($row + 1) * $maxHt) + 6}]
	    if {$hgt < [$self cget -minheight]} {
		set hgt [$self cget -minheight]
	    }
	    $_interior configure \
		-width [expr {$leftExt + ($width - $rightExt) + (2 * [$_interior cget -borderwidth])+ 6}] \
		-height $hgt
	}

	if {$repeat} {
	    after idle [list $self _packToolbar]
	}
    }

} ;# End of ::toolbar
